@@ -24,7 +24,6 @@ import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Set;
-import java.util.Stack;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
@@ -36,14 +35,14 @@ import javax.management.NotificationEmitter;
 import javax.management.NotificationFilter;
 import javax.management.NotificationListener;
 import javax.management.ObjectName;
-import javax.servlet.MultipartConfigElement;
-import javax.servlet.Servlet;
-import javax.servlet.ServletConfig;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletException;
-import javax.servlet.SingleThreadModel;
-import javax.servlet.UnavailableException;
-import javax.servlet.annotation.MultipartConfig;
+
+import jakarta.servlet.MultipartConfigElement;
+import jakarta.servlet.Servlet;
+import jakarta.servlet.ServletConfig;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.UnavailableException;
+import jakarta.servlet.annotation.MultipartConfig;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.ContainerServlet;
@@ -69,7 +68,6 @@ import org.apache.tomcat.util.modeler.Util;
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  */
-@SuppressWarnings("deprecation") // SingleThreadModel
 public class StandardWrapper extends ContainerBase implements ServletConfig, Wrapper, NotificationEmitter {
 
     private final Log log = LogFactory.getLog(StandardWrapper.class); // must not be static
@@ -103,13 +101,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     protected long available = 0L;
 
     /**
-     * The broadcaster that sends EE notifications.
+     * The broadcaster that sends j2ee notifications.
      */
     protected final NotificationBroadcasterSupport broadcaster;
 
     /**
-     * The count of allocations that are currently active (even if they are for the same instance, as will be true on a
-     * non-STM servlet).
+     * The count of allocations that are currently active.
      */
     protected final AtomicInteger countAllocated = new AtomicInteger(0);
 
@@ -173,48 +170,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     protected String servletClass = null;
 
 
-    /**
-     * Does this servlet implement the SingleThreadModel interface?
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    protected volatile boolean singleThreadModel = false;
-
-
     /**
      * Are we unloading our servlet instance at the moment?
      */
     protected volatile boolean unloading = false;
 
 
-    /**
-     * Maximum number of STM instances.
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    protected int maxInstances = 20;
-
-
-    /**
-     * Number of instances currently loaded for a STM servlet.
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    protected int nInstances = 0;
-
-
-    /**
-     * Stack containing the STM instances.
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    protected Stack<Servlet> instancePool = null;
-
-
     /**
      * Wait time for servlet unload in ms.
      */
@@ -239,7 +200,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     protected boolean swallowOutput = false;
 
     // To support jmx attributes
-    StandardWrapperValve swValve;
+    protected StandardWrapperValve swValve;
     protected long loadTime = 0;
     protected int classLoadTime = 0;
 
@@ -284,12 +245,26 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         this.overridable = overridable;
     }
 
+    /**
+     * Return the available date/time for this servlet, in milliseconds since the epoch. If this date/time is
+     * Long.MAX_VALUE, it is considered to mean that unavailability is permanent and any request for this servlet will
+     * return an SC_NOT_FOUND error. If this date/time is in the future, any request for this servlet will return an
+     * SC_SERVICE_UNAVAILABLE error. If it is zero, the servlet is currently available.
+     */
     @Override
     public long getAvailable() {
         return this.available;
     }
 
 
+    /**
+     * Set the available date/time for this servlet, in milliseconds since the epoch. If this date/time is
+     * Long.MAX_VALUE, it is considered to mean that unavailability is permanent and any request for this servlet will
+     * return an SC_NOT_FOUND error. If this date/time is in the future, any request for this servlet will return an
+     * SC_SERVICE_UNAVAILABLE error.
+     *
+     * @param available The new available date/time
+     */
     @Override
     public void setAvailable(long available) {
         long oldAvailable = this.available;
@@ -303,14 +278,16 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
 
     /**
-     * @return the number of active allocations of this servlet, even if they are all for the same instance (as will be
-     *             true for servlets that do not implement <code>SingleThreadModel</code>.
+     * @return the number of active allocations of this servlet.
      */
     public int getCountAllocated() {
         return this.countAllocated.get();
     }
 
 
+    /**
+     * @return the load-on-startup order value (negative value means load on first call).
+     */
     @Override
     public int getLoadOnStartup() {
 
@@ -326,6 +303,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Set the load-on-startup order value (negative value means load on first call).
+     *
+     * @param value New load-on-startup value
+     */
     @Override
     public void setLoadOnStartup(int value) {
 
@@ -361,34 +343,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return maximum number of instances that will be allocated when a single thread model servlet is used.
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    public int getMaxInstances() {
-        return this.maxInstances;
-    }
-
-
-    /**
-     * Set the maximum number of instances that will be allocated when a single thread model servlet is used.
-     *
-     * @param maxInstances New value of maxInstances
-     *
-     * @deprecated This will be removed in Tomcat 10.1 onwards.
-     */
-    @Deprecated
-    public void setMaxInstances(int maxInstances) {
-
-        int oldMaxInstances = this.maxInstances;
-        this.maxInstances = maxInstances;
-        support.firePropertyChange("maxInstances", oldMaxInstances, this.maxInstances);
-
-    }
-
-
     /**
      * Set the parent Container of this Wrapper, but only if it is a Context.
      *
@@ -409,12 +363,20 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * @return the run-as identity for this servlet.
+     */
     @Override
     public String getRunAs() {
         return this.runAs;
     }
 
 
+    /**
+     * Set the run-as identity for this servlet.
+     *
+     * @param runAs New run-as identity value
+     */
     @Override
     public void setRunAs(String runAs) {
 
@@ -425,12 +387,20 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * @return the fully qualified servlet class name for this servlet.
+     */
     @Override
     public String getServletClass() {
         return this.servletClass;
     }
 
 
+    /**
+     * Set the fully qualified servlet class name for this servlet.
+     *
+     * @param servletClass Servlet class name
+     */
     @Override
     public void setServletClass(String servletClass) {
 
@@ -456,17 +426,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    @Deprecated
-    public Boolean isSingleThreadModel() {
-        // If the servlet has been loaded either singleThreadModel will be true
-        // or instance will be non-null
-        if (singleThreadModel || instance != null) {
-            return Boolean.valueOf(singleThreadModel);
-        }
-        return null;
-    }
-
-
+    /**
+     * @return <code>true</code> if the Servlet has been marked unavailable.
+     */
     @Override
     public boolean isUnavailable() {
 
@@ -490,7 +452,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         instance = loadServlet();
 
         Class<? extends Servlet> servletClazz = instance.getClass();
-        if (!javax.servlet.http.HttpServlet.class.isAssignableFrom(servletClazz)) {
+        if (!jakarta.servlet.http.HttpServlet.class.isAssignableFrom(servletClazz)) {
             return DEFAULT_SERVLET_METHODS;
         }
 
@@ -508,20 +470,15 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
             for (int i = 0; methods != null && i < methods.length; i++) {
                 Method m = methods[i];
 
-                switch (m.getName()) {
-                    case "doGet":
-                        allow.add("GET");
-                        allow.add("HEAD");
-                        break;
-                    case "doPost":
-                        allow.add("POST");
-                        break;
-                    case "doPut":
-                        allow.add("PUT");
-                        break;
-                    case "doDelete":
-                        allow.add("DELETE");
-                        break;
+                if (m.getName().equals("doGet")) {
+                    allow.add("GET");
+                    allow.add("HEAD");
+                } else if (m.getName().equals("doPost")) {
+                    allow.add("POST");
+                } else if (m.getName().equals("doPut")) {
+                    allow.add("PUT");
+                } else if (m.getName().equals("doDelete")) {
+                    allow.add("DELETE");
                 }
             }
         }
@@ -530,12 +487,18 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * @return the associated servlet instance.
+     */
     @Override
     public Servlet getServlet() {
         return instance;
     }
 
 
+    /**
+     * Set the associated servlet instance.
+     */
     @Override
     public void setServlet(Servlet servlet) {
         instance = servlet;
@@ -544,6 +507,10 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     // --------------------------------------------------------- Public Methods
 
+    /**
+     * Execute a periodic task, such as reloading, etc. This method will be invoked inside the classloading context of
+     * this container. Unexpected throwables will be caught and logged.
+     */
     @Override
     public synchronized void backgroundProcess() {
         super.backgroundProcess();
@@ -567,7 +534,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      */
     public static Throwable getRootCause(ServletException e) {
         Throwable rootCause = e;
-        Throwable rootCauseCheck;
+        Throwable rootCauseCheck = null;
         // Extra aggressive rootCause finding
         int loops = 0;
         do {
@@ -594,6 +561,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Add a new servlet initialization parameter for this servlet.
+     *
+     * @param name  Name of this initialization parameter to add
+     * @param value Value of this initialization parameter to add
+     */
     @Override
     public void addInitParameter(String name, String value) {
 
@@ -608,6 +581,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Add a mapping associated with the Wrapper.
+     *
+     * @param mapping The new wrapper mapping
+     */
     @Override
     public void addMapping(String mapping) {
 
@@ -624,6 +602,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Add a new security role reference record to the set of records for this servlet.
+     *
+     * @param name Role name used within this servlet
+     * @param link Role name used within the web application
+     */
     @Override
     public void addSecurityReference(String name, String link) {
 
@@ -638,6 +622,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Allocate an initialized instance of this Servlet that is ready to have its <code>service()</code> method called.
+     *
+     * @exception ServletException if the servlet init() method threw an exception
+     * @exception ServletException if a loading error occurs
+     */
     @Override
     public Servlet allocate() throws ServletException {
 
@@ -648,111 +638,62 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
         boolean newInstance = false;
 
-        // If not SingleThreadedModel, return the same instance every time
-        if (!singleThreadModel) {
-            // Load and initialize our instance if necessary
-            if (instance == null || !instanceInitialized) {
-                synchronized (this) {
-                    if (instance == null) {
-                        try {
-                            if (log.isTraceEnabled()) {
-                                log.trace("Allocating non-STM instance");
-                            }
-
-                            // Note: We don't know if the Servlet implements
-                            // SingleThreadModel until we have loaded it.
-                            instance = loadServlet();
-                            newInstance = true;
-                            if (!singleThreadModel) {
-                                // For non-STM, increment here to prevent a race
-                                // condition with unload. Bug 43683, test case
-                                // #3
-                                countAllocated.incrementAndGet();
-                            }
-                        } catch (ServletException e) {
-                            throw e;
-                        } catch (Throwable e) {
-                            ExceptionUtils.handleThrowable(e);
-                            throw new ServletException(sm.getString("standardWrapper.allocate"), e);
-                        }
-                    }
-                    if (!instanceInitialized) {
-                        initServlet(instance);
-                    }
-                }
-            }
-
-            if (singleThreadModel) {
-                if (newInstance) {
-                    // Have to do this outside of the sync above to prevent a
-                    // possible deadlock
-                    synchronized (instancePool) {
-                        instancePool.push(instance);
-                        nInstances++;
-                    }
-                }
-            } else {
-                if (log.isTraceEnabled()) {
-                    log.trace("  Returning non-STM instance");
-                }
-                // For new instances, count will have been incremented at the
-                // time of creation
-                if (!newInstance) {
-                    countAllocated.incrementAndGet();
-                }
-                return instance;
-            }
-        }
-
-        synchronized (instancePool) {
-            while (countAllocated.get() >= nInstances) {
-                // Allocate a new instance if possible, or else wait
-                if (nInstances < maxInstances) {
+        // Load and initialize our instance if necessary
+        if (instance == null || !instanceInitialized) {
+            synchronized (this) {
+                if (instance == null) {
                     try {
-                        instancePool.push(loadServlet());
-                        nInstances++;
+                        if (log.isDebugEnabled()) {
+                            log.debug("Allocating instance");
+                        }
+                        instance = loadServlet();
+                        newInstance = true;
+                        // Increment here to prevent a race condition
+                        // with unload. Bug 43683, test case #3
+                        countAllocated.incrementAndGet();
                     } catch (ServletException e) {
                         throw e;
-                    } catch (Throwable t) {
-                        ExceptionUtils.handleThrowable(t);
-                        throw new ServletException(sm.getString("standardWrapper.allocate"), t);
-                    }
-                } else {
-                    try {
-                        instancePool.wait();
-                    } catch (InterruptedException e) {
-                        // Ignore
+                    } catch (Throwable e) {
+                        ExceptionUtils.handleThrowable(e);
+                        throw new ServletException(sm.getString("standardWrapper.allocate"), e);
                     }
                 }
+                if (!instanceInitialized) {
+                    initServlet(instance);
+                }
             }
-            if (log.isTraceEnabled()) {
-                log.trace("  Returning allocated STM instance");
-            }
-            countAllocated.incrementAndGet();
-            return instancePool.pop();
         }
+
+        if (log.isTraceEnabled()) {
+            log.trace("  Returning instance");
+        }
+        // For new instances, count will have been incremented at the
+        // time of creation
+        if (!newInstance) {
+            countAllocated.incrementAndGet();
+        }
+        return instance;
     }
 
 
+    /**
+     * Decrement the allocation count for this servlet.
+     *
+     * @param servlet The servlet to be returned
+     *
+     * @exception ServletException if a deallocation error occurs
+     */
     @Override
     public void deallocate(Servlet servlet) throws ServletException {
-
-        // If not SingleThreadModel, no action is required
-        if (!singleThreadModel) {
-            countAllocated.decrementAndGet();
-            return;
-        }
-
-        // Unlock and free this instance
-        synchronized (instancePool) {
-            countAllocated.decrementAndGet();
-            instancePool.push(servlet);
-            instancePool.notify();
-        }
-
+        countAllocated.decrementAndGet();
     }
 
 
+    /**
+     * Return the value for the specified initialization parameter name, if any; otherwise return <code>null</code>.
+     *
+     * @param name Name of the requested initialization parameter
+     */
     @Override
     public String findInitParameter(String name) {
 
@@ -766,6 +707,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Return the names of all defined initialization parameters for this servlet.
+     */
     @Override
     public String[] findInitParameters() {
 
@@ -779,6 +723,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Return the mappings associated with this wrapper.
+     */
     @Override
     public String[] findMappings() {
 
@@ -792,9 +739,15 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Return the security role link for the specified security role reference name, if any; otherwise return
+     * <code>null</code>.
+     *
+     * @param name Security role reference used within this servlet
+     */
     @Override
     public String findSecurityReference(String name) {
-        String reference;
+        String reference = null;
 
         referencesLock.readLock().lock();
         try {
@@ -817,6 +770,10 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Return the set of security role reference names associated with this servlet, if any; otherwise return a
+     * zero-length array.
+     */
     @Override
     public String[] findSecurityReferences() {
 
@@ -831,12 +788,17 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
 
     /**
-     * {@inheritDoc}
+     * Load and initialize an instance of this servlet, if there is not already at least one initialized instance. This
+     * can be used, for example, to load servlets that are marked in the deployment descriptor to be loaded at server
+     * startup time.
      * <p>
      * <b>IMPLEMENTATION NOTE</b>: Servlets whose classnames begin with <code>org.apache.catalina.</code> (so-called
      * "container" servlets) are loaded by the same classloader that loaded this class, rather than the classloader for
      * the current web application. This gives such classes access to Catalina internals, which are prevented for
      * classes loaded for web applications.
+     *
+     * @exception ServletException if the servlet init() method threw an exception
+     * @exception ServletException if some other loading problem occurs
      */
     @Override
     public synchronized void load() throws ServletException {
@@ -860,18 +822,18 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
             try {
                 jspMonitorON = new ObjectName(oname.toString());
-                Registry.getRegistry(null).registerComponent(instance, jspMonitorON, null);
-            } catch (Exception e) {
-                log.warn(sm.getString("standardWrapper.jspMonitorError", instance), e);
+                Registry.getRegistry(null, null).registerComponent(instance, jspMonitorON, null);
+            } catch (Exception ex) {
+                log.warn(sm.getString("standardWrapper.jspMonitorError", instance));
             }
         }
     }
 
 
     /**
-     * Load and initialize an instance of this servlet, if there is not already at least one initialized instance. This
-     * can be used, for example, to load servlets that are marked in the deployment descriptor to be loaded at server
-     * startup time.
+     * Load and initialize an instance of this servlet, if there is not already an initialized instance. This can be
+     * used, for example, to load servlets that are marked in the deployment descriptor to be loaded at server startup
+     * time.
      *
      * @return the loaded Servlet instance
      *
@@ -880,7 +842,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     public synchronized Servlet loadServlet() throws ServletException {
 
         // Nothing to do if we already have an instance or an instance pool
-        if (!singleThreadModel && (instance != null)) {
+        if (instance != null) {
             return instance;
         }
 
@@ -905,19 +867,19 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                 unavailable(null);
                 // Restore the context ClassLoader
                 throw new ServletException(sm.getString("standardWrapper.notServlet", servletClass), e);
-            } catch (Throwable t) {
-                Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
-                ExceptionUtils.handleThrowable(throwable);
+            } catch (Throwable e) {
+                e = ExceptionUtils.unwrapInvocationTargetException(e);
+                ExceptionUtils.handleThrowable(e);
                 unavailable(null);
 
                 // Added extra log statement for Bugzilla 36630:
                 // https://bz.apache.org/bugzilla/show_bug.cgi?id=36630
                 if (log.isDebugEnabled()) {
-                    log.debug(sm.getString("standardWrapper.instantiate", servletClass), throwable);
+                    log.debug(sm.getString("standardWrapper.instantiate", servletClass), e);
                 }
 
                 // Restore the context ClassLoader
-                throw new ServletException(sm.getString("standardWrapper.instantiate", servletClass), throwable);
+                throw new ServletException(sm.getString("standardWrapper.instantiate", servletClass), e);
             }
 
             if (multipartConfigElement == null) {
@@ -936,13 +898,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
             classLoadTime = (int) (System.currentTimeMillis() - t1);
 
-            if (servlet instanceof SingleThreadModel) {
-                if (instancePool == null) {
-                    instancePool = new Stack<>();
-                }
-                singleThreadModel = true;
-            }
-
             initServlet(servlet);
 
             fireContainerEvent("load", this);
@@ -951,7 +906,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         } finally {
             if (swallowOutput) {
                 String log = SystemLogHandler.stopCapture();
-                if (log != null && !log.isEmpty()) {
+                if (log != null && log.length() > 0) {
                     if (getServletContext() != null) {
                         getServletContext().log(log);
                     } else {
@@ -967,7 +922,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     private synchronized void initServlet(Servlet servlet) throws ServletException {
 
-        if (instanceInitialized && !singleThreadModel) {
+        if (instanceInitialized) {
             return;
         }
 
@@ -1006,6 +961,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         }
     }
 
+    /**
+     * Remove the specified initialization parameter from this servlet.
+     *
+     * @param name Name of the initialization parameter to remove
+     */
     @Override
     public void removeInitParameter(String name) {
 
@@ -1020,6 +980,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Remove a mapping associated with the wrapper.
+     *
+     * @param mapping The pattern to remove
+     */
     @Override
     public void removeMapping(String mapping) {
 
@@ -1036,6 +1001,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Remove any security role reference for the specified role name.
+     *
+     * @param name Security role used within this servlet to be removed
+     */
     @Override
     public void removeSecurityReference(String name) {
 
@@ -1050,6 +1020,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Process an UnavailableException, marking this servlet as unavailable for the specified amount of time.
+     *
+     * @param unavailable The exception that occurred, or <code>null</code> to mark this servlet as permanently
+     *                        unavailable
+     */
     @Override
     public void unavailable(UnavailableException unavailable) {
         getServletContext().log(sm.getString("standardWrapper.unavailable", getName()));
@@ -1068,17 +1044,23 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Unload all initialized instances of this servlet, after calling the <code>destroy()</code> method for each
+     * instance. This can be used, for example, prior to shutting down the entire servlet engine, or prior to reloading
+     * all of the classes from the Loader associated with our Loader's repository.
+     *
+     * @exception ServletException if an exception is thrown by the destroy() method
+     */
     @Override
     public synchronized void unload() throws ServletException {
 
         // Nothing to do if we have never loaded the instance
-        if (!singleThreadModel && (instance == null)) {
+        if (instance == null) {
             return;
         }
         unloading = true;
 
         // Loaf a while if the current instance is allocated
-        // (possibly more than once if non-STM)
         if (countAllocated.get() > 0) {
             int nRetries = 0;
             long delay = unloadDelay / 20;
@@ -1116,8 +1098,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
             } catch (Throwable t) {
                 t = ExceptionUtils.unwrapInvocationTargetException(t);
                 ExceptionUtils.handleThrowable(t);
-                instancePool = null;
-                nInstances = 0;
                 fireContainerEvent("unload", this);
                 unloading = false;
                 throw new ServletException(sm.getString("standardWrapper.destroyException", getName()), t);
@@ -1134,7 +1114,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                 // Write captured output
                 if (swallowOutput) {
                     String log = SystemLogHandler.stopCapture();
-                    if (log != null && !log.isEmpty()) {
+                    if (log != null && log.length() > 0) {
                         if (getServletContext() != null) {
                             getServletContext().log(log);
                         } else {
@@ -1151,42 +1131,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         instance = null;
 
         if (isJspServlet && jspMonitorON != null) {
-            Registry.getRegistry(null).unregisterComponent(jspMonitorON);
+            Registry.getRegistry(null, null).unregisterComponent(jspMonitorON);
         }
 
-        if (singleThreadModel && (instancePool != null)) {
-            try {
-                while (!instancePool.isEmpty()) {
-                    Servlet s = instancePool.pop();
-                    if (Globals.IS_SECURITY_ENABLED) {
-                        try {
-                            SecurityUtil.doAsPrivilege("destroy", s);
-                        } finally {
-                            SecurityUtil.remove(s);
-                        }
-                    } else {
-                        s.destroy();
-                    }
-                    // Annotation processing
-                    if (!((Context) getParent()).getIgnoreAnnotations()) {
-                        ((StandardContext) getParent()).getInstanceManager().destroyInstance(s);
-                    }
-                }
-            } catch (Throwable t) {
-                t = ExceptionUtils.unwrapInvocationTargetException(t);
-                ExceptionUtils.handleThrowable(t);
-                instancePool = null;
-                nInstances = 0;
-                unloading = false;
-                fireContainerEvent("unload", this);
-                throw new ServletException(sm.getString("standardWrapper.destroyException", getName()), t);
-            }
-            instancePool = null;
-            nInstances = 0;
-        }
-
-        singleThreadModel = false;
-
         unloading = false;
         fireContainerEvent("unload", this);
     }
@@ -1195,12 +1142,21 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     // -------------------------------------------------- ServletConfig Methods
 
 
+    /**
+     * @return the initialization parameter value for the specified name, if any; otherwise return <code>null</code>.
+     *
+     * @param name Name of the initialization parameter to retrieve
+     */
     @Override
     public String getInitParameter(String name) {
         return findInitParameter(name);
     }
 
 
+    /**
+     * @return the set of initialization parameter names defined for this servlet. If none are defined, an empty
+     *             Enumeration is returned.
+     */
     @Override
     public Enumeration<String> getInitParameterNames() {
 
@@ -1214,6 +1170,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * @return the servlet context with which this servlet is associated.
+     */
     @Override
     public ServletContext getServletContext() {
         if (parent == null) {
@@ -1226,6 +1185,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * @return the name of this servlet.
+     */
     @Override
     public String getServletName() {
         return getName();
@@ -1243,10 +1205,28 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         return swValve.getMinTime();
     }
 
+    /**
+     * Returns the number of requests processed by the wrapper.
+     *
+     * @return the number of requests processed by the wrapper.
+     *
+     * @deprecated The return type will change to long in Tomcat 11 onwards. Callers of this method should switch to
+     *                 storing the result of calls to this method in a long value rather than an int.
+     */
+    @Deprecated
     public int getRequestCount() {
         return swValve.getRequestCount();
     }
 
+    /**
+     * Returns the number of requests processed by the wrapper that resulted in an error.
+     *
+     * @return the number of requests processed by the wrapper that resulted in an error.
+     *
+     * @deprecated The return type will change to long in Tomcat 11 onwards. Callers of this method should switch to
+     *                 storing the result of calls to this method in a long value rather than an int.
+     */
+    @Deprecated
     public int getErrorCount() {
         return swValve.getErrorCount();
     }
@@ -1305,7 +1285,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     protected Method[] getAllDeclaredMethods(Class<?> c) {
 
-        if (c.equals(javax.servlet.http.HttpServlet.class)) {
+        if (c.equals(jakarta.servlet.http.HttpServlet.class)) {
             return null;
         }
 
@@ -1339,7 +1319,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected void startInternal() throws LifecycleException {
+    protected synchronized void startInternal() throws LifecycleException {
 
         // Send j2ee.state.starting notification
         if (this.getObjectName() != null) {
@@ -1369,7 +1349,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected void stopInternal() throws LifecycleException {
+    protected synchronized void stopInternal() throws LifecycleException {
 
         setAvailable(Long.MAX_VALUE);
 
@@ -1393,12 +1373,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         if (this.getObjectName() != null) {
             Notification notification = new Notification("j2ee.state.stopped", this.getObjectName(), sequenceNumber++);
             broadcaster.sendNotification(notification);
-
-            // Send j2ee.object.deleted notification
-            notification = new Notification("j2ee.object.deleted", this.getObjectName(), sequenceNumber++);
-            broadcaster.sendNotification(notification);
         }
 
+        // Send j2ee.object.deleted notification
+        Notification notification = new Notification("j2ee.object.deleted", this.getObjectName(), sequenceNumber++);
+        broadcaster.sendNotification(notification);
+
     }
 
 
@@ -1467,6 +1447,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Remove a JMX notificationListener
+     *
+     * @see javax.management.NotificationEmitter#removeNotificationListener(javax.management.NotificationListener,
+     *          javax.management.NotificationFilter, java.lang.Object)
+     */
     @Override
     public void removeNotificationListener(NotificationListener listener, NotificationFilter filter, Object object)
             throws ListenerNotFoundException {
@@ -1475,8 +1461,15 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     protected MBeanNotificationInfo[] notificationInfo;
 
+    /**
+     * Get JMX Broadcaster Info
+     *
+     * @see javax.management.NotificationBroadcaster#getNotificationInfo()
+     */
     @Override
     public MBeanNotificationInfo[] getNotificationInfo() {
+        // FIXME: we not send j2ee.state.failed
+        // FIXME: we not send j2ee.attribute.changed
         if (notificationInfo == null) {
             notificationInfo = new MBeanNotificationInfo[] {
                     new MBeanNotificationInfo(new String[] { "j2ee.object.created" }, Notification.class.getName(),
@@ -1496,6 +1489,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Add a JMX-NotificationListener
+     *
+     * @see javax.management.NotificationBroadcaster#addNotificationListener(javax.management.NotificationListener,
+     *          javax.management.NotificationFilter, java.lang.Object)
+     */
     @Override
     public void addNotificationListener(NotificationListener listener, NotificationFilter filter, Object object)
             throws IllegalArgumentException {
@@ -1503,6 +1502,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
+    /**
+     * Remove a JMX-NotificationListener
+     *
+     * @see javax.management.NotificationBroadcaster#removeNotificationListener(javax.management.NotificationListener)
+     */
     @Override
     public void removeNotificationListener(NotificationListener listener) throws ListenerNotFoundException {
         broadcaster.removeNotificationListener(listener);
